from GlassWindow import GlassWindow
from Observable import Observable
import configvalues as conf
import desktop
import values

import commands
import os
import sys
import gtk
import gobject



#
# Class for the calendar window.
#
class CalWindow(GlassWindow, Observable):

    # messages to the observers
    OBS_WHEEL = 0
    OBS_CLOSE = 1
    OBS_MOVE = 2
    OBS_MENU = 3
    OBS_CLICK = 4
    OBS_JUMP_TO_MONTH = 5
    

    #
    # Constructor.
    #
    def __init__(self, config):

        # whether to use EWMH flags
        self.__use_ewmh = int(config.get([conf.TEMPORARY, conf.USE_EWMH]))

        # flag indicating whether this window is currently being dragged
        self.__is_dragging = 0
        self.__drag_offset = (0, 0)

        # remember the current window position to detect movements
        self.__position = (0, 0)


    	GlassWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
        import icon; self.set_icon(icon.ICON)
        self.set_title(values.NAME)
        
        self.connect("delete-event", self.__on_close)
        self.connect("scroll-event", self.__on_mouse_wheel)
        self.connect("key-press-event", self.__on_key)
        self.connect("configure-event", self.__on_configure)

        # window moving
        self.connect("button-press-event", self.__on_button_press)
        self.connect("button-release-event", self.__on_button_release)

        self.add_events(gtk.gdk.SCROLL_MASK | gtk.gdk.POINTER_MOTION_MASK |
                        gtk.gdk.BUTTON_PRESS_MASK |
                        gtk.gdk.BUTTON_RELEASE_MASK)

        # layout
        self.__layout = gtk.Fixed()
        self.__layout.show()
        self.add(self.__layout)

        self.set_decorated(False)
        self.stick()

        if (self.__use_ewmh):
            # keep window below others
            self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
            self.set_keep_below(True)

            gobject.idle_add(self.set_property, "skip-taskbar-hint", 1)
            gobject.idle_add(self.set_property, "skip-pager-hint", 1)

        self.connect("leave-notify-event", self.__on_leave)


    #
    # Reacts on leaving the window.
    #
    def __on_leave(self, src, event):
    
        if (not self.__is_dragging):
            if (self.__use_ewmh):
                self.set_keep_above(False)
                self.set_keep_below(True)



    #
    # Reacts on mouse wheel.
    #
    def __on_mouse_wheel(self, src, event):

        if (event.direction == gtk.gdk.SCROLL_UP): step = -1
        elif (event.direction == gtk.gdk.SCROLL_DOWN): step = 1
        else: step = 0

        if (step != 0):
            self.update_observer(self.OBS_WHEEL, step)



    #
    # Reacts on key presses.
    #
    def __on_key(self, src, event):

        key = gtk.gdk.keyval_name(event.keyval)

        # SPACE updates the background
        if (key == "space"):
            self.update_bg()

        elif (key == "Page_Up"):
            self.update_observer(self.OBS_WHEEL, -1)

        elif (key == "Page_Down"):
            self.update_observer(self.OBS_WHEEL, 1)

        elif (key == "Home"):
            self.update_observer(self.OBS_JUMP_TO_MONTH, 0)

        # F10 opens the menu
        #elif (key == "F10"):
        #    self.update_observer(self.OBS_MENU, 0, event.time)

        else:
            # F1 - F12 navigate through the months of the year
            # (if your window manager traps one of these, hold down
            # a modifier key, e.g. Shift)
            for i in range(1, 13):
                if (key == "F%d" % i):
                    self.update_observer(self.OBS_JUMP_TO_MONTH, i)

    

    #
    # Reacts on moving the window.
    #
    def __on_configure(self, src, event):

        pos = self.window.get_origin()
        if (pos != self.__position):
            self.update_observer(self.OBS_MOVE, pos)
            self.__position = pos




    #
    # Reacts on closing the window.
    #
    def __on_close(self, src, event):

        self.update_observer(self.OBS_CLOSE)



    #
    # Reacts on pressing a mouse button.
    #
    def __on_button_press(self, src, event):

        if (self.__use_ewmh):
            self.set_keep_below(False)
            self.set_keep_above(True)

        button = event.button
        modifiers = event.state
        if (button == 1):
            x, y = self.get_pointer()
            self.update_observer(self.OBS_CLICK, x, y)

        if (button == 2):
            x, y = self.get_pointer()
            self.__is_dragging = 1
            self.__drag_offset = (x, y)
            self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.FLEUR))
            gobject.timeout_add(20, self.__on_move)
        



    #
    # Reacts on releasing a mouse button.
    #
    def __on_button_release(self, src, event):

        if (self.__is_dragging):
            self.__is_dragging = 0
            if (self.__use_ewmh):
                self.set_keep_above(False)
                self.set_keep_below(True)
            self.window.set_cursor(None)
            self.update_bg()

        elif (event.button == 3):
            self.update_observer(self.OBS_MENU, event.button, event.time)



    #
    # Reacts on moving the mouse pointer.
    #
    def __on_move(self):

        if (self.__is_dragging):
            offx, offy = self.__drag_offset
            winx, winy = self.window.get_origin()
            x, y = self.get_pointer()
            x += winx; y += winy
            self.move(x - offx, y - offy)

            return True

        else:
            return False



    #
    # Updates the background.
    #
    def update_bg(self):

        GlassWindow.update_bg(self)
        self.stick()
        


    #
    # Puts or moves a child widget at the given position.
    #
    def put(self, widget, x, y):

        if (not widget in self.__layout.get_children()):
            self.__layout.put(widget, x, y)
        else:
            self.__layout.move(widget, x, y)



    #
    # Switches to window move mode.
    #
    def move_window(self):

        if (self.__use_ewmh):
            self.set_keep_below(False)
            self.set_keep_above(True)

        x, y = self.get_pointer()
        width, height = self.window.get_size()
        x = max(4, min(x, width - 4))
        y = max(4, min(y, height - 4))
        self.__is_dragging = 1
        self.__drag_offset = (x, y)
        gobject.timeout_add(20, self.__on_move)

        



    #
    # Configures the window.
    #
    def set_config(self, config):

        x = config.get([conf.GEOMETRY, conf.WINDOW, conf.X])
        y = config.get([conf.GEOMETRY, conf.WINDOW, conf.Y])

        width = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW, conf.WIDTH])
        height = config.get([conf.SKIN, conf.GEOMETRY, conf.WINDOW,
                             conf.HEIGHT])

        self.show()


        self.move(x, y)
        self.resize(width, height)
        self.set_size_request(width, height)
        
        self.update_bg()
