from Observable import Observable
from code.ListBox import ListBox
from code import values

import gtk
from gtk import glade
import gobject
import os



#
# List of ical calendars.
#
class CalendarList(Observable):

    OBS_ADD = 0
    OBS_REMOVE = 1

    __CMD_NEW = 0
    __CMD_REMOVE = 1


    #
    # Constructor.
    #
    def __init__(self):

        # handler for the close action
        self.__close_handler = None
        
        # list of ical files
        self.__calendars = []
        
        
        gui = glade.XML(os.path.join(os.environ[values.DESKCALHOME],
                                     values.GLADEFILE))
        self.__window = gui.get_widget("calendars")
                
        from code import icon; self.__window.set_icon(icon.ICON)
        self.__window.set_title(_("Overlayed calendars"))
        self.__window.connect("delete-event", self.__on_close)

        # close button
        gui.get_widget("btn_close_cal").connect("clicked", self.__on_close)

        # list of appointments
        self.__listbox = ListBox(["Calendar file"])
        #self.__listbox.set_headers_visible(True)
        self.__listbox.connect_select_row(self.__on_select)
        self.__listbox.show()
        gui.get_widget("gap_calbox").add(self.__listbox)

        # buttons
        btn = gui.get_widget("btn_add_cal")
        #btn.set_label(_("New Appointment"))
        btn.connect("clicked", self.__on_button, self.__CMD_NEW)

        btn = gui.get_widget("btn_remove_cal")
        #btn.set_label(_("Remove Appointment"))
        btn.connect("clicked", self.__on_button, self.__CMD_REMOVE)


    #
    # Sets the list of calendars.
    #
    def set_calendars(self, cals):
    
        self.__calendars = cals
        self.__listbox.clear()
        for c in cals:
            self.__listbox.append(c)


    #
    # Returns the list of calendars.
    #
    def get_calendars(self):
    
        return self.__calendars
        

    def show(self):
    
        self.__window.show_all()



    #
    # Reacts on closing the window.
    #
    def __on_close(self, *args):
 
        self.__window.hide()
        if (self.__close_handler):
            self.__close_handler()

        return True



    #
    # Reacts on selecting a calendar.
    #
    def __on_select(self, src, row):

        pass


    #
    # Reacts on pressing a command button.
    #
    def __on_button(self, src, cmd):

        if (cmd == self.__CMD_NEW):
            dialog = gtk.FileChooserDialog(_("Select a calendar file"),
                                           None,
                                           gtk.FILE_CHOOSER_ACTION_OPEN,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
            if (dialog.run() == gtk.RESPONSE_OK):
                cal = dialog.get_filename()
                self.__add_calendar(cal)
                self.update_observer(self.OBS_ADD, cal)
            dialog.destroy()

        elif (cmd == self.__CMD_REMOVE):
            row = self.__listbox.get_cursor()
            cal = self.__calendars[row]
            del self.__calendars[row]
            self.set_calendars(self.__calendars)
            self.update_observer(self.OBS_REMOVE, cal)
            
            
    #
    # Adds the given calendar file.
    #
    def __add_calendar(self, cal):
    
        self.__calendars.append(cal)
        self.__listbox.append(cal)



    #
    # Connects an action handler to the close action.
    #
    def connect_close(self, handler):

        self.__close_handler = handler
